﻿//
// Copyright (C) 2011, ZAO Nanosoft. All rights reserved.
//
namespace HostMgdReversi
{
  using System;
  using System.Collections.Generic;
  using System.Linq;
  using System.Text;

 #if NCAD
    using HostMgd.ApplicationServices;
    using HostMgd.EditorInput;
    
    using Teigha.DatabaseServices;
    using Teigha.Runtime;
    using Teigha.Geometry;
    
    using Platform = Teigha;
    using Host = HostMgd;
 #else
    using Autodesk.AutoCAD.DatabaseServices;
    using Autodesk.AutoCAD.Runtime;
    using Autodesk.AutoCAD.Geometry;
    using Autodesk.AutoCAD.ApplicationServices;
    using Autodesk.AutoCAD.EditorInput;
    
    using Platform = Autodesk.AutoCAD;
    using Host = Autodesk.AutoCAD;
  #endif
  /// <summary>
  /// Main assembly class
  /// </summary>
  public class HostMgdReversi
  {
    /// <summary>
    /// Command to start the game
    /// </summary>
    [CommandMethod("Play")]
    public void PlayReversiGame()
    {
      var game = new Game();
      game.Init();
      PrepareScreen();
      game.Play();
    }

    private void PrepareScreen()
    {
      Database db = Application.DocumentManager.MdiActiveDocument.Database;
      Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;

      using (Transaction myT = db.TransactionManager.StartTransaction())
      {
        var dict = myT.GetObject(db.VisualStyleDictionaryId, OpenMode.ForRead) as DBDictionary;
        var curVP = ed.GetCurrentView();
        curVP.VisualStyleId = dict.GetAt("Conceptual");
        curVP.RenderMode = ViewTableRecordRenderMode.GouraudShaded;

        db.UpdateExt(true);
        double scrRatio = curVP.Width / curVP.Height;
        Matrix3d W2U = Matrix3d.PlaneToWorld(curVP.ViewDirection);
        W2U = Matrix3d.Displacement(curVP.Target - Point3d.Origin) * W2U;
        W2U = Matrix3d.Rotation(-curVP.ViewTwist, curVP.ViewDirection, curVP.Target) * W2U;
        W2U = W2U.Inverse();
        Point3d extmin = db.Extmin;
        Point3d extmax = db.Extmax;
        Extents3d extents = new Extents3d(extmin, extmax);
        extents.TransformBy(W2U);
        double width = (extents.MaxPoint.X - extents.MinPoint.X);
        double height = (extents.MaxPoint.Y - extents.MinPoint.Y);
        Point2d center = new Point2d((extents.MaxPoint.X + extents.MinPoint.X) * 0.5, (extents.MaxPoint.Y + extents.MinPoint.Y) * 0.5);

        if (width > (height * scrRatio))
        {
          height = width / scrRatio;
        }
        curVP.Height = height;
        curVP.Width = height * scrRatio;
        curVP.CenterPoint = center;
        ed.SetCurrentView(curVP);
       myT.Commit();
      }
    }
  }
}